@@ -1,7 +1,7 @@
 """If the following ENV VAR were available, many end-to-end test cases would run.
-LAB_APP_CLIENT_SECRET=...
 LAB_OBO_CLIENT_SECRET=...
 LAB_APP_CLIENT_ID=...
+LAB_APP_CLIENT_CERT_PFX_PATH=...
 LAB_OBO_PUBLIC_CLIENT_ID=...
 LAB_OBO_CONFIDENTIAL_CLIENT_ID=...
 """
@@ -507,12 +507,8 @@ class LabBasedTestCase(E2eTestCase):
     @classmethod
     def setUpClass(cls):
         # https://docs.msidlab.com/accounts/apiaccess.html#code-snippet
-        try:
-            cls.session = get_session(get_lab_app(), ["https://msidlab.com/.default"])
-        except LabTokenError:
-            cls.session = get_session(get_lab_app(), [
-                # A lab change since June 10, 2024 which may or may not be reverted
-                "https://request.msidlab.com/.default",
+        cls.session = get_session(get_lab_app(), [
+                "https://request.msidlab.com/.default",  # A lab change since June 10, 2024
             ])
 
     @classmethod
